import greenfoot.Actor;

public abstract class Body extends Actor {

    private class Vector {
    
        private double x;
        private double y;
    
        public Vector() {
            set(0, 0);
        }
    
        public Vector(Vector vector) {
            set(vector);
        }
    
        public Vector(double x, double y) {
            set(x, y);
        }
    
        public void set(Vector vector) {
            set(vector.getX(), vector.getY());
        }
    
        public void set(double x, double y) {
            this.x = x;
            this.y = y;
        }
    
        public void add(Vector vector) {
            add(vector.getX(), vector.getY());
        }
    
        public void add(double addX, double addY) {
            x += addX;
            y += addY;
        }
    
        public void subtract(Vector vector) {
            subtract(vector.getX(), vector.getY());
        }
    
        public void subtract(double subtractX, double subtractY) {
            x -= subtractX;
            y -= subtractY;
        }
    
        public void multiply(double magnitude) {
            x *= magnitude;
            y *= magnitude;
        }
    
        public void divide(double magnitude) {
            x /= magnitude;
            y /= magnitude;
        }
    
        public void tilt(double tilt) {
            double angle = getAngle();
            double magnitude = getLength();
    
            x = (double) (magnitude * Math.cos(angle + tilt));
            y = (double) (magnitude * Math.sin(angle + tilt));
        }
    
        public double normalize() {
            double magnitude = getLength();
    
            if(magnitude != 0.0    ) {
                divide(magnitude);
            }
    
            return magnitude;
        }
    
        public double getNorm() {
            return (double) Math.hypot(x, y);
        }
    
        public double getDistance(Vector vector) {
            return getDistance(vector.getX(), vector.getY());
        }
    
        public double getDistance(double pointX, double pointY) {
            double deltaX = x - pointX;
            double deltaY = y - pointX;
    
            return (double) Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
        }
    
        public double getDot(Vector vector) {
            return getDot(vector.getX(), vector.getY());
        }
    
        public double getDot(double dotX, double dotY) {
            return (x * dotX) + (y * dotY);
        }
    
        public double getLength() {
            return (double) Math.sqrt((x * x) + (y * y));
        }
    
        public double getAngle() {
            return (double) Math.atan2(y, x);
        }
    
        public double getX() {
            return x;
        }
    
        public double getY() {
            return y;
        }
    }
    
    private boolean fixed;
    private double mass;
    private double damping;
    private Vector gravity;
    private Vector forces;
    private Vector velocity;
    
    public Body() {
        gravity = new Vector();
        forces = new Vector();
        velocity = new Vector();
    }
    
    @Override
    public void act() {
        if(fixed) {
            return;
        }
    
        addForce(gravity.getX(), gravity.getY());
    
        velocity.add(forces);
        velocity.multiply(damping);
    
        setLocation((int) (getX() + velocity.getX()), (int) (getY() + velocity.getY()));
    
        forces.set(0, 0);
    }
    
    public void clearForces() {
        gravity.set(0, 0);
        forces.set(0, 0);
        velocity.set(0, 0);
    }
    
    public void setFixed(boolean fixed) {
        this.fixed = fixed;
    }
    
    public void setMass(double mass) {
        this.mass = mass;
    }
    
    public void setDamping(double damping) {
        this.damping = 1. - clamp(damping);
    }
    
    public void addGravity(double gravityX, double gravityY) {
        gravity.add(gravityX, gravityY);
    }
    
    public void setGravity(double gravityX, double gravityY) {
        gravity.set(gravityX, gravityY);
    }
    
    public void addForce(double forceX, double forceY) {
        forces.add(forceX * mass, forceY * mass);
    }
    
    public void setForces(double forceX, double forceY) {
        forces.set(forceX * mass, forceY * mass);
    }
    
    public void addVelocity(double velocityX, double velocityY) {
        velocity.add(velocityX, velocityY);
    }
    
    public void setVelocity(double velocityX, double velocityY) {
        velocity.set(velocityX, velocityY);
    }
    
    private double clamp(double value) {
        return value > 1.0 ? 1.0 : value < 0.0 ? 0.0 : value;
    }
    
    public boolean isFixed() {
        return fixed;
    }
    
    public double getMass() {
        return mass;
    }
    
    public double getDamping() {
        return damping;
    }
    
    public Vector getGravity() {
        return gravity;
    }
    
    public Vector getForces() {
        return forces;
    }
    
    public Vector getVelocity() {
        return velocity;
    }
}