import java.awt.Color;

import greenfoot.Actor;
import greenfoot.Greenfoot;
import greenfoot.GreenfootImage;

public class Particle extends Body {

    private static final GreenfootImage HIDDEN_IMAGE = new GreenfootImage(1, 1);

    private static GreenfootImage colorized;
    private static Color lastColor;

    private Color color;
    private GreenfootImage image;
    private boolean active;
    private boolean scaleable;
    private double life;
    private double fade;
    private double scale;
    private double width;
    private double height;

    public Particle() {
        setImage(HIDDEN_IMAGE);
    }

    @Override
    public void act() {
        if(active) {
            super.act();

            if(fade > 0.0) {
                life -= fade;
            }
            if(scaleable && scale > 0.0) {
                scale -= fade;

                int scaledWidth = (int) (width * scale);
                int scaledHeight = (int) (height * scale);
                if(scaledWidth > 0.0 && scaledHeight > 0.0) {
                    getImage().scale(scaledWidth, scaledHeight);
                }
            }
            if(life <= 0.0 || scaleable && (width < 0.0 || height < 0.0)) {
                active = false;

                setImage(HIDDEN_IMAGE);
            }
        }
    }

    public void setColor(Color color) {
        this.color = color;
    }

    public void setParticleImage(GreenfootImage image) {
        this.image = image;

        if(color != null && color != lastColor) {
            colorized = colorize(image, color);
            lastColor = color;
        }

        setImage(new GreenfootImage(colorized));

        width = getImage().getWidth();
        height = getImage().getHeight();
    }

    public void setScaleable(boolean scaleable) {
        this.scaleable = scaleable;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public void setLife(double life) {
        this.life = life;
    }

    public void setFade(double fade) {
        this.fade = fade;
    }

    public void setScale(double scale) {
        this.scale = scale;
    }

    private GreenfootImage colorize(GreenfootImage image, Color color) {
        GreenfootImage colorized = new GreenfootImage(image);

        for(int x = 0; x < colorized.getWidth(); x++) {
            for(int y = 0; y < colorized.getHeight(); y++) {
                Color pixel = colorized.getColorAt(x, y);

                int red = 255 - (pixel.getRed() - color.getRed());
                int green = 255 - (pixel.getGreen() - color.getGreen());
                int blue = 255 - (pixel.getBlue() - color.getBlue());
                int alpha = pixel.getAlpha();

                red = Math.max(red, 0);
                red = Math.min(255, red);

                green = Math.max(green, 0);
                green = Math.min(255, green);

                blue = Math.max(blue, 0);
                blue = Math.min(255, blue);

                colorized.setColorAt(x, y, new Color(red, green, blue, alpha));
            }
        }

        return colorized;
    }

    public Color getColor() {
        return color;
    }

    public GreenfootImage getParticleImage() {
        return image;
    }

    public boolean isScaleable() {
        return scaleable;
    }

    public boolean isActive() {
        return active;
    }

    public double getLife() {
        return life;
    }

    public double getFade() {
        return fade;
    }

    public double getScale() {
        return scale;
    }
}