import greenfoot.*;

import java.awt.Color;

public class ParticleEffect extends World {

    private static final int MAXIMUM_PARTICLES = 200;
    private static final long COLOR_CHANGE_TIMER = 1000;
    private static final Color[] colors = new Color[] {new Color(255, 127, 127), new Color(255, 191, 127),
            new Color(255, 255, 127), new Color(191, 255, 127), new Color(127, 255, 127), new Color(127, 255, 191),
            new Color(127, 255, 255), new Color(127, 191, 255), new Color(127, 127, 255), new Color(191, 127, 255),
            new Color(255, 127, 255), new Color(255, 127, 191)};


    private ParticleEmitter emitter;
    private long lastColorChange;
    private Color activeColor;

    public ParticleEffect() {
        super(600, 400, 1);
        
        // Initialize the black background
        GreenfootImage backgroundImage = new GreenfootImage(getWidth(), getHeight());
        backgroundImage.setColor(Color.DARK_GRAY);
        backgroundImage.fill();    
   
        setBackground(backgroundImage);

        // Initialize the particle emitter object
        emitter = new ParticleEmitter(this, MAXIMUM_PARTICLES);

        // Load the particle image
        GreenfootImage particleImage = new GreenfootImage("particle.png");

        // Configure the particle emitter
        emitter.setImage(particleImage);
        emitter.setScaleable(true);
        emitter.setLife(0.3, 1.0);
        emitter.setFade(0.009, 0.01);
        emitter.setScale(0.6, 1.0);

        // Configure the physical body of the particles
        emitter.setFixed(false);
        emitter.setMass(0.6, 1.0);
        emitter.setDamping(0.1, 0.2);
        emitter.setForceX(-2.0, 0);
        emitter.setForceY(-2.0, 4.0);
        emitter.setGravityY(0.1, 0.3);

        Greenfoot.start();
    }

    @Override
    public void act() {
        if(System.currentTimeMillis() - lastColorChange > COLOR_CHANGE_TIMER) {
            activeColor = colors[(int) (Math.random() * colors.length - 1)];

            if(activeColor != null) {
                emitter.setColor(activeColor);
            }

            lastColorChange = System.currentTimeMillis();
        }

        MouseInfo mouseInfo = Greenfoot.getMouseInfo();
        if(mouseInfo == null) {
            return;
        }

        int mouseX = mouseInfo.getX();
        int mouseY = mouseInfo.getY();

        emitter.emit(mouseX, mouseY, 2);
    }
}