import java.awt.Color;

import greenfoot.World;
import greenfoot.GreenfootImage;

public class ParticleEmitter {

    private class Interval {

        private double min;
        private double max;

        public Interval() {
            set(0, 0);
        }

        public Interval(Interval interval) {
            set(interval);
        }

        public Interval(double min, double max) {
            set(min, max);
        }

        public void set(Interval interval) {
            set(interval.getMin(), interval.getMax());
        }

        public void set(double min, double max) {
            this.min = min;
            this.max = max;
        }

        public void setMin(double min) {
            this.min = min;
        }

        public void setMax(double max) {
            this.max = max;
        }

        public double getMin() {
            return min;
        }

        public double getMax() {
            return max;
        }
    }

    private GreenfootImage image;
    private int offset;
    private Particle[] particles;
    private Color color;
    private boolean scaleable;
    private boolean fixed;
    private Interval life;
    private Interval fade;
    private Interval scale;
    private Interval mass;
    private Interval damping;
    private Interval forceX;
    private Interval forceY;
    private Interval gravityX;
    private Interval gravityY;

    public ParticleEmitter(World world, int maxParticles) {
        life = new Interval();
        fade = new Interval();
        scale = new Interval();
        mass = new Interval();
        damping = new Interval();
        forceX = new Interval();
        forceY = new Interval();
        gravityX = new Interval();
        gravityY = new Interval();

        particles = new Particle[maxParticles];
        for(int i = 0; i < particles.length; i++) {
            particles[i] = new Particle();

            world.addObject(particles[i], 0, 0);
        }
    }

    public void emitAll(int x, int y) {
        emit(x, y, particles.length);
    }

    public void emit(int x, int y, int amount) {
        for(int i = 0; i < amount; i++) {
            emit(x, y);
        }
    }

    public void emit(int x, int y) {
        if(offset == particles.length - 1) {
            offset = 0;
        }

        Particle particle = particles[offset++];
        if(!particle.isActive()) {
            particle.setActive(true);

            particle.setColor(color);
            particle.setScaleable(scaleable);
            particle.setParticleImage(image);
            particle.setLife(getRandom(life.getMin(), life.getMax()));
            particle.setFade(getRandom(fade.getMin(), fade.getMax()));
            particle.setScale(getRandom(scale.getMin(), scale.getMax()));

            particle.clearForces();
            particle.setFixed(fixed);
            particle.setGravity(getRandom(gravityX.getMin(), gravityX.getMax()),
                 getRandom(gravityY.getMin(), gravityY.getMax()));
            particle.setMass(getRandom(mass.getMin(), mass.getMax()));
            particle.setDamping(getRandom(damping.getMin(), damping.getMax()));
            particle.addForce(getRandom(forceX.getMin(), forceX.getMax()),
                    getRandom(forceY.getMin(), forceY.getMax()));

            particle.setLocation(x, y);
        }
    }

    public void setImage(GreenfootImage image) {
        this.image = image;
    }

    public void setColor(Color color) {
        this.color = color;
    }

    public void setScaleable(boolean scaleable) {
        this.scaleable = scaleable;
    }

    public void setFixed(boolean fixed) {
        this.fixed = fixed;
    }

    public void setLife(double minLife, double maxLife) {
        life.set(minLife, maxLife);
    }

    public void setFade(double minFade, double maxFade) {
        fade.set(minFade, maxFade);
    }

    public void setScale(double minScale, double maxScale) {
        scale.set(minScale, maxScale);
    }

    public void setMass(double minMass, double maxMass) {
        mass.set(minMass, maxMass);
    }

    public void setDamping(double minDamping, double maxDamping) {
        damping.set(minDamping, maxDamping);
    }

    public void setForceX(double minForceX, double maxForceX) {
        forceX.set(minForceX, maxForceX);
    }

    public void setForceY(double minForceY, double maxForceY) {
        forceY.set(minForceY, maxForceY);
    }

    public void setGravityX(double minGravityX, double maxGravityX) {
        gravityX.set(minGravityX, maxGravityX);
    }

    public void setGravityY(double minGravityY, double maxGravityY) {
        gravityY.set(minGravityY, maxGravityY);
    }

    private double getRandom(double min, double max) {
        return min + ((double) (Math.random() * (max - min)));
    }

    public GreenfootImage getImage() {
        return image;
    }

    public Color getColor() {
        return color;
    }

    public boolean isScaleable() {
        return scaleable;
    }

    public boolean isFixed() {
        return fixed;
    }

    public Interval getLife() {
        return life;
    }

    public Interval getFade() {
        return fade;
    }

    public Interval getScale() {
        return scale;
    }

    public Interval getMass() {
        return mass;
    }

    public Interval getDamping() {
        return damping;
    }

    public Interval getForceX() {
        return forceX;
    }

    public Interval getForceY() {
        return forceY;
    }

    public Interval getGravityX() {
        return gravityX;
    }

    public Interval getGravityY() {
        return gravityY;
    }
}